Class: Syntropy::HTTP::ServerConnection

Inherits:
Object
  • Object
show all
Defined in:
lib/syntropy/http/server_connection.rb

Overview

Implements an HTTP/1.1 connection received by the Syntropy server. This implementation rejects incoming HTTP/0.9 or HTTP/1.0 requests. The response body is sent exclusively using chunked transfer encoding. Request bodies are accepted using either fixed length (Content-Length header) or chunked transfer encoding.

Constant Summary collapse

SEND_FLAGS =
UM::MSG_NOSIGNAL | UM::MSG_WAITALL
EMPTY_CHUNK =
"0\r\n\r\n"
EMPTY_CHUNK_LEN =
EMPTY_CHUNK.bytesize
CHUNKED_ENCODING_POSTLUDE =
"\r\n#{EMPTY_CHUNK}"

Instance Attribute Summary collapse

Instance Method Summary collapse

Constructor Details

#initialize(machine, fd, env, io_mode: :socket, &app) ⇒ ServerConnection

Returns a new instance of ServerConnection.



16
17
18
19
20
21
22
23
24
25
26
# File 'lib/syntropy/http/server_connection.rb', line 16

def initialize(machine, fd, env, io_mode: :socket, &app)
  @machine = machine
  @fd = fd
  @env = env
  @logger = env[:logger]
  @io = machine.io(fd, io_mode)
  @app = app

  @done = nil
  @response_headers = nil
end

Instance Attribute Details

#fdObject (readonly)

Returns the value of attribute fd.



14
15
16
# File 'lib/syntropy/http/server_connection.rb', line 14

def fd
  @fd
end

#loggerObject (readonly)

Returns the value of attribute logger.



14
15
16
# File 'lib/syntropy/http/server_connection.rb', line 14

def logger
  @logger
end

#response_headersObject (readonly)

Returns the value of attribute response_headers.



14
15
16
# File 'lib/syntropy/http/server_connection.rb', line 14

def response_headers
  @response_headers
end

Instance Method Details

#closeObject



242
243
244
245
246
247
248
# File 'lib/syntropy/http/server_connection.rb', line 242

def close
  return if @closed

  @closed = true
  @machine.shutdown(@fd, UM::SHUT_WR)
  @machine.close_async(@fd)
end

#complete?(req) ⇒ Boolean

Returns:

  • (Boolean)


113
114
115
# File 'lib/syntropy/http/server_connection.rb', line 113

def complete?(req)
  req.headers[':body-done-reading']
end

#finish(request) ⇒ void

This method returns an undefined value.

Finishes the response to the current request. If no headers were sent, default headers are sent using #send_headers.



203
204
205
206
207
208
209
210
# File 'lib/syntropy/http/server_connection.rb', line 203

def finish(request)
  request.tx_incr(EMPTY_CHUNK_LEN)
  @machine.send(@fd, EMPTY_CHUNK, EMPTY_CHUNK_LEN, SEND_FLAGS)
  return if @done

  @logger&.info(request, request, response_headers: @response_headers)
  @done = true
end

#get_body(req) ⇒ Object



95
96
97
98
99
100
101
102
# File 'lib/syntropy/http/server_connection.rb', line 95

def get_body(req)
  headers = req.headers
  return nil if headers[':body-done-reading']

  body = @io.http_read_body(headers)
  headers[':body-done-reading'] = true if body
  body
end

#get_body_chunk(req) ⇒ Object



104
105
106
107
108
109
110
111
# File 'lib/syntropy/http/server_connection.rb', line 104

def get_body_chunk(req)
  headers = req.headers
  return nil if headers[':body-done-reading']

  chunk = @io.http_read_body_chunk(headers)
  headers[':body-done-reading'] = true if !chunk
  chunk
end

#handle_error(request, err) ⇒ void

This method returns an undefined value.

Handles an error encountered while serving a request by logging the error and optionally sending an error response with the relevant HTTP status code. For I/O errors, no response is sent.

Parameters:



70
71
72
73
74
75
76
77
78
79
80
81
82
83
84
# File 'lib/syntropy/http/server_connection.rb', line 70

def handle_error(request, err)
  case err
  when SystemCallError
    log_error(err, 'I/O error')
    false
  when ProtocolError
    log_error(err, err.message)
    respond(request, err.message, ':status' => err.http_status)
  else
    log_error(err, 'Internal error')
    return if !request || @done

    respond(request, 'Internal server error', ':status' => INTERNAL_SERVER_ERROR)
  end
end

#log_error(err, message) ⇒ void

This method returns an undefined value.

Logs the given err and given message.

Parameters:

  • err (Exception)

    error

  • message (String)

    error message



91
92
93
# File 'lib/syntropy/http/server_connection.rb', line 91

def log_error(err, message)
  @logger&.error(message: "#{message}, closing connection", error: err)
end

#respond(request, body, headers) ⇒ Object

Sends response including headers and body. Waits for the request to complete if not yet completed. The body is sent using chunked transfer encoding.

Parameters:

  • request (Syntropy::Request)

    HTTP request

  • body (String)

    response body

  • headers


150
151
152
153
154
155
156
157
158
159
160
161
162
163
164
# File 'lib/syntropy/http/server_connection.rb', line 150

def respond(request, body, headers)
  headers = @response_headers.merge(headers) if @response_headers

  formatted_headers = format_headers(headers, body)
  @response_headers = headers
  request&.tx_incr(formatted_headers.bytesize + (body ? body.bytesize : 0))
  if body
    chunk_prelude = "#{body.bytesize.to_s(16)}\r\n"
    @machine.sendv(@fd, formatted_headers, chunk_prelude, body, CHUNKED_ENCODING_POSTLUDE)
  else
    @machine.send(@fd, formatted_headers, formatted_headers.bytesize, SEND_FLAGS)
  end
  @logger&.info(request: request, response_headers: headers) if request
  @done = true
end

#respond_with_static_file(req, path, env, cache_headers) ⇒ Object



212
213
214
215
216
217
218
219
220
221
222
223
224
225
226
227
228
229
230
231
232
233
234
235
236
237
238
239
240
# File 'lib/syntropy/http/server_connection.rb', line 212

def respond_with_static_file(req, path, env, cache_headers)
  fd = @machine.open(path, UM::O_RDONLY)
  env ||= {}
  if env[:headers]
    env[:headers].merge!(cache_headers)
  else
    env[:headers] = cache_headers
  end

  maxlen = env[:max_len] || 65_536
  buf = String.new(capacity: maxlen)
  headers_sent = nil
  loop do
    res = @machine.read(fd, buf, maxlen, 0)
    if res < maxlen && !headers_sent
      return respond(req, buf, env[:headers])
    elsif res == 0
      return finish(req)
    end

    if !headers_sent
      send_headers(req, env[:headers])
      headers_sent = true
    end
    done = res < maxlen
    send_chunk(req, buf, done: done)
    return if done
  end
end

#runObject



28
29
30
31
32
33
34
35
36
37
38
39
40
41
42
43
44
# File 'lib/syntropy/http/server_connection.rb', line 28

def run
  loop do
    @done = nil
    @response_headers = nil
    persist = serve_request
    break if !persist
  end
rescue UM::Terminate
  # server is terminated, do nothing
rescue StandardError => e
  @logger&.error(
    message:  'Uncaught error while running connection',
    error:    e
  )
ensure
  @machine.close_async(@fd)
end

#send_chunk(request, chunk, done: false) ⇒ void

This method returns an undefined value.

Sends a response body chunk. If no headers were sent, default headers are sent using #send_headers. if the done option is true(thy), an empty chunk will be sent to signal response completion to the client.

Parameters:

  • request (Syntropy::Request)

    HTTP request

  • chunk (String)

    response body chunk

  • done (boolean) (defaults to: false)

    whether the response is completed



186
187
188
189
190
191
192
193
194
195
196
197
198
# File 'lib/syntropy/http/server_connection.rb', line 186

def send_chunk(request, chunk, done: false)
  data = +''
  data << "#{chunk.bytesize.to_s(16)}\r\n#{chunk}\r\n" if chunk
  data << EMPTY_CHUNK if done
  return if data.empty?

  request.tx_incr(data.bytesize)
  @machine.send(@fd, data, data.bytesize, SEND_FLAGS)
  return if @done || !done

  @logger&.info(request: request, response_headers: @response_headers)
  @done = true
end

#send_headers(request, headers, empty_response: false) ⇒ void

This method returns an undefined value.

Sends response headers. If empty_response is truthy, the response status code will default to 204, otherwise to 200.

Parameters:

  • request (Syntropy::Request)

    HTTP request

  • headers (Hash)

    response headers

  • empty_response (boolean) (defaults to: false)

    whether a response body will be sent



172
173
174
175
176
177
# File 'lib/syntropy/http/server_connection.rb', line 172

def send_headers(request, headers, empty_response: false)
  formatted_headers = format_headers(headers, !empty_response)
  request.tx_incr(formatted_headers.bytesize)
  @machine.send(@fd, formatted_headers, formatted_headers.bytesize, SEND_FLAGS)
  @response_headers = headers
end

#serve_requestObject

Processes an incoming request by parsing the headers, creating a request object and handing it off to the app handler. Returns true if the connection should be persisted.



49
50
51
52
53
54
55
56
57
58
59
60
61
# File 'lib/syntropy/http/server_connection.rb', line 49

def serve_request
  @closed = nil
  headers = parse_headers
  return false if !headers

  request = Syntropy::Request.new(headers, self)

  @app.call(request)
  persist_connection?(headers)
rescue StandardError => e
  handle_error(request, e)
  false
end


129
130
131
132
133
134
135
136
# File 'lib/syntropy/http/server_connection.rb', line 129

def set_cookie(*cookies)
  existing_cookies = @response_headers && @response_headers['Set-Cookie']
  if existing_cookies
    @response_headers['Set-Cookie'] = existing_cookies + cookies
  else
    set_response_headers('Set-Cookie' => cookies)
  end
end

#set_response_headers(headers) ⇒ void

This method returns an undefined value.

Sets response headers before sending any response. This method is used to add headers such as Set-Cookie or cache control headers to a response before actually responding, specifically in middleware hooks.

Parameters:

  • headers (Hash)

    response headers



125
126
127
# File 'lib/syntropy/http/server_connection.rb', line 125

def set_response_headers(headers)
  @response_headers ? @response_headers.merge!(headers) : @response_headers = headers
end

#with_stream {|@io, @fd| ... } ⇒ Object

Yields:

  • (@io, @fd)


250
251
252
# File 'lib/syntropy/http/server_connection.rb', line 250

def with_stream
  yield @io, @fd
end